import honeyshare.config
from honeyshare.api_requests import make_request


class APICommon:
    def __init__(self, key=None):
        self.key = key or config.KEY

    def get_request(self, path, page_num=None, page_size=None, metadata=False):
        req = make_request(path, self.key, page_num, page_size)

        if req.status_code == 403:
            raise ExNotAuthenticated
        elif req.status_code != 200:
            raise ExUnknownError(req.status_code)

        try:
            js = req.json()
        except:
            raise ExCannotParseJSON

        if metadata:
            return APIResponse(js)

        try:
            return js["Result"]
        except KeyError as e:
            raise ExResponseMalformed(e)


class APIResponse:
    def __init__(self, js):
        try:
            self.endpoint = js["Endpoint"]
            self.result = js["Result"]

            if "page_size" in js:
                self.page_size = js["PageSize"]

            if "page_num" in js:
                self.page_num = js["PageNum"]
        except KeyError as e:
            raise ExResponseMalformed(e)


class ExNotAuthenticated(Exception):
    def __init__(self):
        super().__init__("Not authenticated")


class ExUnknownError(Exception):
    def __init__(self, code):
        super().__init__(f"Unknown Error. Code: {code}")


class ExCannotParseJSON(Exception):
    def __init__(self):
        super().__init__("Cannot Parse JSON")


class ExResponseMalformed(Exception):
    def __init__(self, key_error):
        missing = key_error.args[0]
        super().__init__(f"Response is Malformed. Missing: {missing}")



from honeyshare.api.api_common import APICommon
from honeyshare.api.util import ensureAttr


class ExPortNeeded(Exception):
    def __init__(self):
        super().__init__("Port needed for operation")


class Port(APICommon):
    def __call__(
        self,
        port: str = None,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        self._port = port
        return self

    def list(self, page_num: int = None, page_size: int = None, metadata: bool = False):
        return self.get_request("/ports", page_num, page_size, metadata)

    @ensureAttr("_port", ExPortNeeded)
    def port(self, metadata: bool = False):
        return self.get_request(f"/ports/{self._port}", metadata=metadata)

    @ensureAttr("_port", ExPortNeeded)
    def ipv4(
        self,
        ipv4: str = None,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        if ipv4 is None:
            return self.get_request(
                f"/ports/{self._port}/ipv4", page_num, page_size, metadata=metadata
            )
        return self.get_request(f"/ports/{self._port}/ipv4/{ipv4}", metadata=metadata)

    @ensureAttr("_port", ExPortNeeded)
    def bytes(
        self,
        ipv4: str,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        return self.get_request(
            f"/ports/{self._port}/ipv4/{ipv4}/bytes",
            page_num,
            page_size,
            metadata=metadata,
        )



def ensureAttr(attr, exc):
    def _ensureAttr(func):
        def decorator(self, *args, **kwargs):
            if not hasattr(self, attr) or getattr(self, attr) is None:
                raise exc
            return func(self, *args, **kwargs)

        return decorator

    return _ensureAttr



from honeyshare.api.api_common import APICommon


class Blacklist(APICommon):
    def ipv4s(self):
        return self.get_request("/blacklist/ipv4s")

    def hostnames(self):
        return self.get_request("/blacklist/hostnames")



from honeyshare.api.api_common import APICommon
from honeyshare.api.util import ensureAttr


class ExHostnameNeeded(Exception):
    def __init__(self):
        super().__init__("Hostname needed for operation")


class Hostname(APICommon):
    def __call__(self, hostname: str = None):
        self._hostname = hostname
        return self

    @ensureAttr("_hostname", ExHostnameNeeded)
    def hostname(self, metadata: bool = False):
        return self.get_request(f"/hostnames/{self._hostname}", metadata=metadata)

    def list(self, page_num: int = None, page_size: int = None, metadata: bool = False):
        return self.get_request(
            "/hostnames", page_num=page_num, page_size=page_size, metadata=metadata
        )

    @ensureAttr("_hostname", ExHostnameNeeded)
    def ipv4(
        self,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        return self.get_request(
            f"/hostnames/{self._hostname}/ipv4",
            page_num=page_num,
            page_size=page_size,
            metadata=metadata,
        )



from honeyshare.api.api_common import APICommon


class Timeseries(APICommon):
    def list(self, page_num: int = None, page_size: int = None, metadata: bool = False):
        return self.get_request(
            "/timeseries", page_num=page_num, page_size=page_size, metadata=metadata
        )



from abc import ABC

from honeyshare import config, api


class HoneyShare:
    def __init__(self, key=None):
        self.key = key or config.KEY
        self.Blacklist = api.Blacklist(self.key)
        self.IPv4 = api.IPv4(self.key)
        self.Hostname = api.Hostname(self.key)
        self.Port = api.Port(self.key)
        self.Timeseries = api.Timeseries(self.key)



import requests

from honeyshare import config


def make_url(path):
    return f"https://{config.HOSTNAME}/{config.API_BASE}{path}"


def make_request(path, key, page_num, page_size):
    url = make_url(path)

    params = {}
    if page_num is not None:
        params["pagenum"] = page_num
    if page_size is not None:
        params["pagesize"] = page_size

    resp = requests.get(url, headers={config.HEADER: key}, params=params)
    return resp



from honeyshare.api.api_common import APICommon
from honeyshare.api.util import ensureAttr


class ExIPv4IPNeeded(Exception):
    def __init__(self):
        super().__init__("IPv4 needed for operation")


class IPv4(APICommon):
    def __call__(self, ipv4: str = None):
        self._ipv4 = ipv4
        return self

    def list(self, page_num: int = None, page_size: int = None, metadata: bool = False):
        return self.get_request("/ipv4", page_num, page_size, metadata)

    @ensureAttr("_ipv4", ExIPv4IPNeeded)
    def ipv4(self, metadata: bool = False):
        return self.get_request(f"/ipv4/{self._ipv4}", metadata=metadata)

    @ensureAttr("_ipv4", ExIPv4IPNeeded)
    def ports(
        self,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        return self.get_request(
            f"/ipv4/{self._ipv4}/ports",
            page_num=page_num,
            page_size=page_size,
            metadata=metadata,
        )

    @ensureAttr("_ipv4", ExIPv4IPNeeded)
    def hostnames(
        self,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        return self.get_request(
            f"/ipv4/{self._ipv4}/hostnames",
            page_num=page_num,
            page_size=page_size,
            metadata=metadata,
        )

    @ensureAttr("_ipv4", ExIPv4IPNeeded)
    def timeseries(
        self,
        page_num: int = None,
        page_size: int = None,
        port: str = None,
        metadata: bool = False,
    ):
        if port is None:
            return self.get_request(
                f"/ipv4/{self._ipv4}/timeseries",
                page_num=page_num,
                page_size=page_size,
                metadata=metadata,
            )
        return self.get_request(
            f"/ipv4/{self._ipv4}/ports/{port}/timeseries",
            page_num=page_num,
            page_size=page_size,
            metadata=metadata,
        )

    @ensureAttr("_ipv4", ExIPv4IPNeeded)
    def bytes(
        self,
        port: str,
        page_num: int = None,
        page_size: int = None,
        metadata: bool = False,
    ):
        return self.get_request(
            f"/ipv4/{self._ipv4}/ports/{port}/bytes",
            page_num=page_num,
            page_size=page_size,
            metadata=metadata,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['honeyshare', 'honeyshare.api']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.32.4,<3.0.0']

setup_kwargs = {
    'name': 'honeyshare',
    'version': '1.0.0',
    'description': 'Python API client for HoneyShare.live',
    'long_description': '# HoneyShareLib\n\nThe [HoneyShare](https://honeyshare.live/) API Client.\n\n## Installing\n\nAvailable in [PYPI](https://pypi.org/project/honeyshare/).\n\n    pip install honeyshare\n\n## Usage\n\nInitialize HoneyShare with a [Key](https://honeyshare.live/licenses):\n\n    from honeyshare import HoneyShare\n\n    hs = HoneyShare(key="Your HoneyShare Key"")\n\nThe library is organized around the five HoneyShare objects:\n\n    hs.Blacklist.ipv4s()     // Blacklists of IPs\n    hs.Blacklist.hostnames() // Blacklists of Hostnames\n\n    hs.IPv4.list()               // IP list\n    hs.IPv4(ip).ipv4()           // IP\'s meta data\n    hs.IPv4(ip).ports()          // Ports accessed by IP\n    hs.IPv4(ip).hostnames()      // Hostnames of IP\n    hs.IPv4(ip).timeseries()     // Timeseries of IP\n    hs.IPv4(ip).timeseries(port) // Timeseries of IP on Port\n    hs.IPv4(ip).bytes(port)      // Bytes sent by IP on Port\n\n    hs.Hostname.list()               // Hostname list\n    hs.Hostname(hostname).hostname() // Hostname\'s meta data\n    hs.Hostname(hostname).ipv4()     // IPs of Hostname\n\n    hs.Port.list()          // List of Ports\n    hs.Port(port).port()    // Port\'s meta data\n    hs.Port(port).ipv4()    // IP\'s that acceed port\n    hs.Port(port).ipv4(ip)  // IP\'s meta data on Port\n    hs.Port(port).bytes(ip) // Bytes sent by IP on Port\n\n    hs.Timeseries.list() // List all connections\n\n## Building and Installing Locally\n\n    poetry build\n    pip install --force-reinstall dist/honeyshare-*.whl\n',
    'author': 'Pedro Melgueira',
    'author_email': 'pedromelgueira@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from honeyshare.api.blacklist import Blacklist
from honeyshare.api.ipv4 import IPv4
from honeyshare.api.hostname import Hostname
from honeyshare.api.port import Port
from honeyshare.api.timeseries import Timeseries
